.model large
.code
public _allocate             ;allocate memory
public _mfree                ;free memory
public _period		     ;Shows the end of the period
public _openr                ;Open a file to read
public _openw                ;Open a file to write
public _closefile            ;Close an opened file
public _read                 ;Read from a file
public _write                ;write to the file
public _ferr                 ;Returns 1 if open error occured
public _zero		     ;Shows the zero points

mem dw 0		     ;allocated memory pointer [seg]
cpoi dw 0                    ;circle buffer pointer [offs]
acm db 0,0,0		     ;accummulator for the averaging
avr db 0		     ;average of the circle buffer
abil db 0		     ;above(1)/belowe(0) flag
cor db 0		     ;is abil initialized?
fileerr db 0                 ;File error flag (error->1)

rdata db 0		     ;old data
upinit db 1		     ;Initial cycle
min dw 80		     ;edge is acceptable after minimum 100 sample
max dw 150		     ;edge must be after 150 sample
dcount dw 0		     ;data counter

_zero proc far		;Shows the second(above) zero point of the function
push bp			;Returns 1 in case of rising edge
mov bp,sp
inc [dcount]
mov ax,[bp+6]		;input data
mov ah,[upinit]
cmp ah,1
jz initz
mov ah,[rdata]		;AL=recent data  AH=old data
mov [rdata],al
cmp ah,128		;old data is below 128
jc ok1			;maybe OK
jmp no1
ok1:
cmp al,128		;recent data is above or equal 128
jc no1			;does not meet

mov ax,[dcount]		;There was a rising edge but after 100 data?
cmp ax,[min]
jc at10
xor ax,ax
mov [dcount],ax
mov ax,1		;OK
jmp vegez
at10:
xor ax,ax
jmp vegez

no1:			;not OK, there was not a rising edge
mov ax,[dcount]
cmp ax,[max]
ja at11
xor ax,ax
jmp vegez
at11:
xor ax,ax
mov [dcount],ax
mov ax,1
jmp vegez

initz:			;initialize
mov [rdata],al
mov al,0
mov [upinit],al
vegez:
pop bp
retf
_zero endp

_period proc far	;IN: next data byte,  OUT: 1->end of a period
push bx                 ;allocated mem=512 byte
push cx
push ds
push bp
mov bp,sp
mov ax,[bp+12]		;save data into circle
mov bx,[mem]
mov ds,bx
mov bx,[cpoi]
mov ds:[bx],al
inc bx			;circling mem poi
and bx,1Fh		;buffer is 512 byte
mov [cpoi],bx

xor ax,ax     		;zero accummulator
mov [acm],al
mov [acm+1],al
mov [acm+2],al

mov cx,512		;computing the average
mov bx,0
aver:
mov al,ds:[bx]
inc bx
add [acm+2],al		;adding to accummulator
adc [acm+1],ah
adc [acm],ah
loop aver

mov ah,[acm]		;divide by 512
mov al,[acm+1]
shr ax,1
mov [avr],al		;save the computed average

mov bl,[abil]
mov ah,[bp+12]		;INPUT data
cmp al,ah
jc above_1
jz above_1
xor ax,ax
mov [abil],al
jmp at_1
above_1:
mov al,1
mov [abil],al
at_1:
cmp bl,al		;is abil changed? (zero place?)
jz noch
mov ax,1
jmp vege
noch:
mov ax,0
vege:

mov bl,[cor]            ;destroy result because of the init?
cmp bl,0
jnz at_2
xor ax,ax
at_2:
mov bl,1
mov [cor],bl

pop bp
pop ds
pop cx
pop bx
retf
_period endp


_allocate proc far           ;Allocate memory
push bp                      ;Returns NULL if error
push bx
mov bp,sp
mov ah,48h
mov bx,[bp+8]                ;size of block in paragraphs
int 21h
jc mhiba1
mov [mem],ax                 ;save pointer
mov dx,ax
xor ax,ax
pop bx
pop bp
retf
mhiba1: xor ax,ax
xor dx,dx
pop bx
pop bp
retf
_allocate endp

_mfree proc far              ;Free memory
push bp
push es
mov bp,sp
mov ax,[bp+10]               ;Blokk cme
mov es,ax
mov ah,49h                   ;Returns 1 if error
int 21h
jc mhiba2
xor ax,ax
pop es
pop bp
retf
mhiba2: mov ax,1
pop es
pop bp
retf
_mfree endp

_openr proc far              ;open a file to read
push ds
push dx
push bp
mov bp,sp
mov ax,[bp+12]               ;filename seg.
mov ds,ax
mov dx,[bp+10]               ;filename offs.
mov ax,3D00h
int 21h                      ;returns: AX=fileno
jc ferr1
mov dl,0
mov [fileerr],dl
jmp op1
ferr1:
mov dl,1
mov [fileerr],dl
op1:
pop bp
pop dx
pop ds
retf
_openr endp

_openw proc far              ;open a file to write
push ds
push dx
push cx
push bp
mov bp,sp
mov ax,[bp+14]               ;filename seg.
mov ds,ax
mov dx,[bp+12]               ;filename offs.
mov ax,3C00h
mov cx,0
int 21h                      ;returns: AX=fileno
jc ferr2
mov dl,0
mov [fileerr],dl
jmp op2
ferr2:
mov dl,1
mov [fileerr],dl
op2:
pop bp
pop cx
pop dx
pop ds
retf
_openw endp

_closefile proc far          ;close an opened file
push bx                      ;AX=1->error
push bp
mov bp,sp
mov ax,3E00h
mov bx,[bp+8]                ;Fileno
int 21h
jc ferr3
mov ax,0
pop bp
pop bx
retf
ferr3:
mov ax,1
pop bp
pop bx
retf
_closefile endp

_read proc far               ;read from a file
push bx
push ds
push dx
push cx
push bp
mov bp,sp
mov bx,[bp+14]               ;Fileno
mov ax,[bp+20]               ;buffer seg.
mov ds,ax
mov dx,[bp+18]               ;buffer offs.
mov cx,[bp+16]               ;size
mov ax,3F00h
int 21h
jc ferr4
mov bx,0
mov [fileerr],bl
jmp rd1
ferr4:
mov bx,1
mov [fileerr],bl
rd1:
pop bp
pop cx
pop dx
pop ds
pop bx
retf
_read endp

_write proc far               ;write to a file
push bx
push ds
push dx
push cx
push bp
mov bp,sp
mov bx,[bp+14]               ;Fileno
mov ax,[bp+20]               ;buffer seg.
mov ds,ax
mov dx,[bp+18]               ;buffer offs.
mov cx,[bp+16]               ;size
mov ax,4000h
int 21h
jc ferr5
mov bx,0
mov [fileerr],bl
jmp rd2
ferr5:
mov bx,1
mov [fileerr],bl
rd2:
pop bp
pop cx
pop dx
pop ds
pop bx
retf
_write endp

_ferr proc far               ;Ask if file open error occured
xor ax,ax
mov al,[fileerr]
retf
_ferr endp

end